"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const assert = require('power-assert');
const sinon = require('sinon');
const testing_1 = require('../testing');
const mongo_entity_1 = require('../mongo_entity');
const lru = require('lru-cache');
const host = process.env.MONGODB_HOST || '127.0.0.1';
const port = process.env.MONGODB_PORT || 27017;
describe('entity', () => {
    it('contains connector and collection name', () => __awaiter(this, void 0, void 0, function* () {
        var connector = yield testing_1.getConnector();
        var collectionName = 'name';
        var entity = new mongo_entity_1.default(connector, collectionName);
        assert.equal(entity.collection.collectionName, collectionName);
        assert.equal(entity.connector, connector);
        assert.notEqual(entity.collection, null);
    }));
    it('is possible to initialise entity with custom cache', function () {
        const cache = { local: '' };
        const entity = new mongo_entity_1.default(null, 'name', cache);
        assert.equal(entity['_dataLoaderOptions'].cacheMap, cache);
        const globalCache = { global: '1' };
        mongo_entity_1.default.DataLoaderOptions = { cacheMap: globalCache };
        const otherEntity = new mongo_entity_1.default(null, 'name');
        assert.equal(otherEntity['_dataLoaderOptions'].cacheMap, globalCache);
        mongo_entity_1.default.DataLoaderOptions = null;
    });
    describe('find', () => {
        it('can find a multiple record', () => __awaiter(this, void 0, void 0, function* () {
            yield testing_1.withEntity((entity) => {
                const find = sinon.spy(entity.collection, 'find');
                const selector = {};
                const fields = {};
                const skip = 1;
                const limit = 2;
                const timeout = 1000;
                entity.find(selector, fields, skip, limit, timeout);
                assert(find.calledWithExactly(selector, fields, skip, limit, timeout));
            });
        }));
    });
    describe('findOne', () => {
        it('can find a single record', () => __awaiter(this, void 0, void 0, function* () {
            yield testing_1.withEntity((entity) => {
                const find = sinon.spy(entity.collection, 'find');
                const selector = {};
                const options = {};
                entity.findOne(selector, options);
                assert(find.calledWithExactly(selector, options));
            });
        }));
    });
    describe('findOneCachedById', () => {
        it('can find and cache results of finding a single item', () => __awaiter(this, void 0, void 0, function* () {
            yield testing_1.withEntity((entity) => __awaiter(this, void 0, void 0, function* () {
                const find = sinon.spy(entity.collection, 'findOne');
                let result = yield entity.findOne({ _id: '1' });
                result = yield entity.findOne({ _id: '1' });
                assert.deepEqual(result, { _id: '1' });
                assert(find.calledTwice);
                find.reset();
                result = yield entity.findOneCachedById('1');
                result = yield entity.findOneCachedById('1');
                assert(find.calledOnce);
            }), {
                entities: [{
                        data: [{ _id: '1' }]
                    }] });
        }));
        it('clears the related cache when a document is updated', function () {
            return __awaiter(this, void 0, void 0, function* () {
                yield testing_1.withEntity((entity) => __awaiter(this, void 0, void 0, function* () {
                    const findSpy = sinon.spy(entity.collection, 'findOne');
                    const cacheSpy = sinon.spy(entity, 'clearUpdateCaches');
                    yield entity.insert({ _id: '1', file: 'foo' });
                    yield entity.insert({ _id: '2', file: 'bar' });
                    let foo = yield entity.findOneCachedById('1');
                    let bar = yield entity.findOneCachedById('2');
                    const selector = { _id: '1' };
                    entity.update(selector, { $set: { file: 'boo' } });
                    assert(cacheSpy.calledWith(selector));
                    findSpy.reset();
                    foo = yield entity.findOneCachedById('1');
                    assert.deepEqual(foo, { _id: '1', file: 'boo' });
                    assert(findSpy.calledOnce);
                    findSpy.reset();
                    bar = yield entity.findOneCachedById('2');
                    assert(findSpy.notCalled);
                }));
            });
        });
        it('clears all caches when a document is updated and selector is unknown', function () {
            return __awaiter(this, void 0, void 0, function* () {
                yield testing_1.withEntity((entity) => __awaiter(this, void 0, void 0, function* () {
                    let foo = yield entity.findOneCachedById('1');
                    let bar = yield entity.findOneCachedById('2');
                    entity.update({ file: 'foo' }, { $set: { file: 'boo' } });
                    const findSpy = sinon.spy(entity.collection, 'findOne');
                    foo = yield entity.findOneCachedById('1');
                    bar = yield entity.findOneCachedById('2');
                    assert(findSpy.calledTwice);
                }));
            });
        });
        it('findOneCached can filter returned results', () => __awaiter(this, void 0, void 0, function* () {
            yield testing_1.withEntity((entity) => __awaiter(this, void 0, void 0, function* () {
                yield entity.insert({ _id: '00', a: '1', b: '2', c: '3' });
                let result = yield entity.findOneCachedById('00', { b: 1 });
                assert.deepEqual(result, { b: '2' });
            }));
        }));
    });
    describe('filter', () => {
        it('can filter objects properties', function () {
            const entity = new mongo_entity_1.default(null, null);
            const obj = { a: 1, b: 2, c: 3 };
            const resultWith = entity.filter(obj, { a: 1, c: 1 });
            assert.deepEqual(resultWith, { a: 1, c: 3 });
            const resultWithout = entity.filter(obj, { a: 0, c: 0 });
            assert.deepEqual(resultWithout, { b: 2 });
        });
        it('does not include undefined elements', function () {
            const entity = new mongo_entity_1.default(null, null);
            const obj = { a: 1, b: 2, c: 3 };
            const resultWith = entity.filter(obj, { a: 1, c: 1, d: 1 });
            assert.deepEqual(resultWith, { a: 1, c: 3 });
        });
        it('throws error when no filter is specified', function () {
            const entity = new mongo_entity_1.default(null, null);
            const obj = {};
            assert.throws(() => entity.filter(obj, {}), /You need to specify the selector!/);
        });
        it('throws error when you mix include exclude', function () {
            const entity = new mongo_entity_1.default(null, null);
            const obj = {};
            assert.throws(() => entity.filter(obj, { a: 1, b: 0 }), /You cannot combine include and exclude!/);
        });
    });
    describe('findManyCached', () => {
        it('can find and cache results of finding multiple items', () => __awaiter(this, void 0, void 0, function* () {
            yield testing_1.withEntity((entity) => __awaiter(this, void 0, void 0, function* () {
                entity.collection.insertOne({ _id: '1' });
                const find = sinon.spy(entity.collection, 'find');
                let result = yield entity.findAllCached();
                result = yield entity.findAllCached();
                assert.deepEqual(result, [{ _id: '1' }]);
                assert(find.calledOnce);
            }));
        }));
        it('findManyCached can filter returned results', () => __awaiter(this, void 0, void 0, function* () {
            yield testing_1.withEntity((entity) => __awaiter(this, void 0, void 0, function* () {
                yield entity.insertMany([
                    { _id: '00', a: '1', b: '2', c: '3' },
                    { _id: '01', a: '1', b: '2', c: '3' }
                ]);
                let result = yield entity.findAllCached({ a: 1, c: 1 });
                assert.deepEqual(result, [{ a: '1', c: '3' }, { a: '1', c: '3' }]);
            }));
        }));
        it('clears the cache when a new document is inserted', function () {
            return __awaiter(this, void 0, void 0, function* () {
                yield testing_1.withEntity((entity) => __awaiter(this, void 0, void 0, function* () {
                    const find = sinon.spy(entity.collection, 'find');
                    entity.insert({ _id: '1' });
                    let result = yield entity.findAllCached();
                    entity.insert({ _id: '2' });
                    result = yield entity.findAllCached();
                    assert(find.calledTwice);
                    assert.deepEqual(result, [{ _id: '1' }, { _id: '2' }]);
                }));
            });
        });
    });
    describe('delete', () => {
        it('can delete a single document', () => __awaiter(this, void 0, void 0, function* () {
            yield testing_1.withEntity((entity) => __awaiter(this, void 0, void 0, function* () {
                yield entity.insert({ _id: '00', a: '1', b: '2', c: '3' });
                yield entity.insert({ _id: '01', a: '1', b: '2', c: '3' });
                let result = yield entity.findAllCached();
                assert.equal(result.length, 2);
                yield entity.delete({ _id: '00' });
                result = yield entity.findAllCached();
                assert.equal(result.length, 1);
            }));
        }));
        it('can delete multiple documents', function () {
            return __awaiter(this, void 0, void 0, function* () {
                yield testing_1.withEntity((entity) => __awaiter(this, void 0, void 0, function* () {
                    yield entity.insert({ _id: '00', a: '1', b: '2', c: '3' });
                    yield entity.insert({ _id: '01', a: '1', b: '2', c: '3' });
                    yield entity.delete({}, true);
                    const result = yield entity.findAllCached();
                    assert.equal(result.length, 0);
                }));
            });
        });
    });
    describe('testing', () => {
        it('can execute test with multiple entities', function () {
            return __awaiter(this, void 0, void 0, function* () {
                yield testing_1.withEntity((e1, e2, e3) => {
                    assert(e1);
                    assert.equal(e1.collection.collectionName, 'e1');
                    assert(e2);
                    assert.equal(e2.collection.collectionName, 'e2');
                    assert(e3);
                    assert.equal(e3.collection.collectionName, 'e3');
                }, { entities: [{ name: 'e1' }, { name: 'e2' }, { name: 'e3' }] });
            });
        });
    });
    describe('custom loaders', () => {
        it('can create a new data-loader with a custom cache', function () {
            return __awaiter(this, void 0, void 0, function* () {
                yield testing_1.withEntity((entity) => {
                    const records = [{ _id: 1, name: 'A' }, { _id: 2, name: 'B' }, { _id: 3, name: 'C' }];
                    entity.insertMany(records);
                    const cache = lru({ max: 2 });
                    const loader = entity.createLoader((name) => {
                        return entity.collection.findOne({ name });
                    }, {
                        cacheMap: cache
                    });
                    const m = yield entity.findOne({ _id: 1 });
                    assert.equal(result, records[2]);
                    const result = yield entity.findOneCached(loader, 'A');
                    assert.equal(result, records[2]);
                });
            });
        });
    });
});
